
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:pc_plugins/custom_exception.dart';

import 'chat_config_template.dart';

class channelsController<C,U,M> extends StateNotifier<List<C>?>{
  final Reader _read;
  final String? uid;
  final tempConfig_chat<C,M,U> config;
  String? lastClearCode;
  StreamSubscription<List<C>>? _channelSub;

  channelsController(this.uid,this._read,{required this.config}) : super(null) {
    _channelSub?.cancel();
  }

  void initStream(){
    if(uid!=null){
      _channelSub=config.stream.listen((event) {
        final selected=_read(config.selectedCPvd);
        if(selected==null){state=event;return;}
        final selectedId=config.getId(selected);

        if(!event.any((e) => config.getId(e)==selectedId)) {
          event.insert(0, selected);}
        if(event.any((e) => config.getId(e)==selectedId
            &&(config.getUread(e,uid!)??0)>=0)){
          final code=_makeClearCode(selected);
          if(lastClearCode!=code){
            lastClearCode=code;
            _read(config.repoPvd).clearUnreadChannel(selectedId);
            event=setRead(event, selectedId);
          }
        }
        state=event;
      },onError: (e,st){debugPrint(e.toString());});}
  }

  void fillUsrs(C v){
    if(state!=null){
      state![state!.indexWhere((e) => config.getId(e)==config.getId(v))]=v;}
  }

  @override
  void dispose() {
    endStream();
    super.dispose();
  }

  void endStream(){
    _channelSub?.cancel();
  }

  String _makeClearCode(C c){
    return config.getId(c)+
        "${config.getLastMsgTime(c)?.toIso8601String()}";
  }

  List<C> setRead(List<C> list,String id){
    if(uid==null)throw const CustomException(message: "uid not found");
    final i=list.indexWhere((e) => config.getId(e)==id);
    return List.of(list..[i]=config.setUread(list[i],uid!,0));
  }

  void selectChannel(C c){
    final cid=config.getId(c);
    if(state!=null&&!state!.any((e) => config.getId(e)==cid)){
      state=state!..insert(0, c);}
    final code=_makeClearCode(c);
    if(lastClearCode!=code){
      lastClearCode=code;
      _read(config.repoPvd).clearUnreadChannel(cid);
      state=setRead(state!, cid);
    }
    _read(config.msgOffsetPvd)[0]=null;
    _read(config.selectedCPvd.state).state=c;
  }

  Future<void> selectChannelid(String cid,
      {M? offset, Function(C)? callback})async{
    _read(config.msgOffsetPvd)[0]=offset;
    if(state!=null&&!(state!.any((e) => config.getId(e)==cid))){
      final c=await _read(config.repoPvd).getChannel(cid,mustAddUsr: true);
      state=state!..insert(0, c);
      _read(config.selectedCPvd.state).state=c;
    }
    else{ _read(config.selectedCPvd.state).state=state!
        .singleWhere((e) => config.getId(e)==cid);}
    if(callback!=null){
      final c=_read(config.selectedCPvd);
      callback(c!);}
  }

  // Future<void> addChannel(List<U>others,String name)async{
  //   others.removeWhere((e) => config.getId(e)==uid);
  //   try{
  //     late T c;
  //     if(others.isEmpty)return;
  //     if(others.length==1)
  //       c = await _read(config.repoPvd).findOrAddChannel(others[0]);
  //     else
  //       c = await _read(config.repoPvd).addChannel(channl(
  //           name: name,
  //           usrs: others..add(me),
  //           unreads: Map.fromIterable(others,key: (e)=>e.id,value: (e)=>0)));
  //
  //     _read(selectedChannelPvd).state=c;
  //   }on CustomException catch(e,st){
  //     _read(itemListExceptionProvider).state=e;
  //   }
  // }
}